#import <Cocoa/Cocoa.h>
#import <IOBluetooth/IOBluetooth.h>
#import "WiiRemoteConstants.h"




/*
	wii remote delegates should respond to one or more of these methods (ideally all of them)
*/
@protocol WiiRemoteDelegateProtocol
	- (void) wiiRemoteDiscoveryError:(NSString *)e;
	- (void) wiiRemoteConnectionError:(NSString *)e;
	- (void) wiiRemoteConnected;
	- (void) wiiRemoteDisconnected;
	- (void) wiiRemoteExpansionPortChanged:(id)remote;
	
	- (void) irPointMovedX:(float)px Y:(float)py wiiRemote:(id)wiiRemote;
	- (void) rawIRData:(IRData[4])irData wiiRemote:(id)wiiRemote;
	- (void) buttonChanged:(WiiButtonType)type isPressed:(BOOL)isPressed wiiRemote:(id)wiiRemote;
	- (void) accelerationChanged:(WiiAccelerationSensorType)type accX:(float)accX accY:(float)accY accZ:(float)accZ wiiRemote:(id)wiiRemote;
	- (void) joyStickChanged:(WiiJoyStickType)type tiltX:(float)tiltX tiltY:(float)tiltY wiiRemote:(id)wiiRemote;
	- (void) analogButtonChanged:(WiiButtonType)type amount:(unsigned)press wiiRemote:(id)wiiRemote;
	- (void) batteryLevelChanged:(float)n;
@end




@interface WiiRemote : NSObject {
	id							delegate;
	BOOL						delRespondsToIR;
	BOOL						delRespondsToRawIR;
	BOOL						delRespondsToButton;
	BOOL						delRespondsToAcc;
	BOOL						delRespondsToJoy;
	BOOL						delRespondsToAButton;
	BOOL						delRespondsToBattery;
	
	//	variables related to the actual bluetooth device- finding & connecting to it
	IOBluetoothDeviceInquiry	*inquiry;		//	the inquiry used to find bluetooth devices
	IOBluetoothDevice			*wiimoteDevice;	//	the wiimote i found (and will connect to)
	IOBluetoothL2CAPChannel		*iChan;			//	the channesl (actually used to send data)
	IOBluetoothL2CAPChannel		*cChan;
	IOBluetoothUserNotification *disconnectNotification;	//	notification- called when device is disconnected
	
	BOOL						readingRegister;
	BOOL						isMotionSensorEnabled;
	BOOL						isIRSensorEnabled;
	BOOL						isVibrationEnabled;
	BOOL						isExpansionPortEnabled;
	BOOL						isExpansionPortAttached;
	BOOL						isLED1Illuminated, isLED2Illuminated, isLED3Illuminated, isLED4Illuminated;
	
	float						lowX;
	float						lowZ;
	int							orientation;
	int							leftPoint;	//	is point 0 or 1 on the left. -1 when not tracking.
	
	WiiExpansionPortType		expType;
	WiiAccCalibData				wiiCalibData;
	WiiAccCalibData				nunchukCalibData;
	WiiJoyStickCalibData		nunchukJoyStickCalibData;
	WiiIRMode					wiiIRMode;
	IRData						irData[4];
	double						batteryLevel;
	
	//	the button values for every button
	BOOL						buttonState[28];
	
	//	wiimote state variables
	unsigned char				accX;
	unsigned char				accY;
	unsigned char				accZ;
	
	//	nunchuk state variables
	unsigned char				nStickX;
	unsigned char				nStickY;
	unsigned char				nAccX;
	unsigned char				nAccY;
	unsigned char				nAccZ;
	
	//	classic controller state variables
	unsigned char				cStickX1;
	unsigned char				cStickY1;
	unsigned char				cStickX2;
	unsigned char				cStickY2;
	unsigned char				cAnalogL;
	unsigned char				cAnalogR;
	
}

//	starts/stops an inquiry (finds remotes)
- (void) startSearching;
- (void) stopSearching;
//	call to connect/disconnect to/from the wiimote device
- (void) connect;
- (void) disconnect;
//	wiimote properties
- (WiiExpansionPortType) expansionPortType;
- (BOOL) isExpansionPortAttached;
- (BOOL) isButtonPressed:(WiiButtonType)type;
- (double) batteryLevel;
- (NSString *) address;
//	setting wiimote capabilities
- (IOReturn) setMotionSensorEnabled:(BOOL)enabled;
- (IOReturn) setForceFeedbackEnabled:(BOOL)enabled;
- (IOReturn) setLEDEnabled1:(BOOL)enabled1 enabled2:(BOOL)enabled2 enabled3:(BOOL)enabled3 enabled4:(BOOL)enabled4;
- (IOReturn) setExpansionPortEnabled:(BOOL)enabled;
- (IOReturn) setIRSensorEnabled:(BOOL)enabled;
//	key-value
- (void) setDelegate:(id)d;
- (id) delegate;

@end




/*
	these are basically "internal" methods- outside apps/objects/etc. shouldn't ever have to call these
*/

@interface WiiRemote (Private)

//	inquiry callbacks (delegate methods)
- (void) deviceInquiryDeviceFound:(IOBluetoothDeviceInquiry *)sender device:(IOBluetoothDevice *)device;
- (void) deviceInquiryComplete:(IOBluetoothDeviceInquiry*)sender error:(IOReturn)error aborted:(BOOL)aborted;
//	these methods are called internally- don't call these manually!
- (void) openConnection;
- (void) performSDPQuery;
- (void) openIChan;
- (void) openCChan;
//	connection-related callback
- (void)disconnected:(IOBluetoothUserNotification*)note fromDevice:(IOBluetoothDevice*)device;
- (void)l2capChannelData:(IOBluetoothL2CAPChannel*)ch data:(void *)d length:(size_t)dataLength;
//	bluetooth data handling
- (IOReturn) writeData:(const unsigned char *)data at:(unsigned long)address length:(size_t)length;
- (IOReturn) readData:(unsigned long)address length:(unsigned short)length;
- (IOReturn) sendCommand:(const unsigned char *)data length:(size_t)length;
- (IOReturn) requestUpdates;
- (void) getCurrentStatus;
//	data parsing
- (void) handleRAMData:(unsigned char *)dp length:(size_t)dataLength;
- (void) handleStatusReport:(unsigned char *)dp length:(size_t)dataLength;
- (void) handleExpansionData:(unsigned char *)dp length:(size_t)dataLength;
- (void) handleIRData:(unsigned char *)dp length:(size_t)dataLength;
- (void) handleButtonReport:(unsigned char *)dp length:(size_t)dataLength;
//	data sending
- (void) sendWiiRemoteButtonEvent:(UInt16)data;
- (void) sendWiiNunchukButtonEvent:(UInt16)data;
- (void) sendWiiClassicControllerButtonEvent:(UInt16)data;

@end
